<!DOCTYPE html>
<html lang="en">

<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>xem</title>
</head>

<body style="background-color: black; color:chartreuse; font-family: 'Courier New', Courier, monospace">
    <br />
    <br />
    <br />
    <center>
        <h1>Test</h1>
    </center>
    <script src="http://127.0.0.1:3939/xem.js?siteId=FOOBAR"></script>
</body>

</html>