<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>xem</title>
</head>
<body style="background-color: black; color:chartreuse; font-family: 'Courier New', Courier, monospace">
    <br/>
    <br/>
    <br/>
    <center><h1>Xem Server</h1></center>
</body>
</html>
